@@ -4,6 +4,18 @@
 #ifndef EXT_H
 #define EXT_H
 
+/* Khronos now provides unified headers for all OpenCL versions, and
+ * it should be included after defining a target OpenCL version
+ * (otherwise, the maximum version will simply be used, but a message
+ * will be printed).
+ *
+ * TODO: until 3.0 gets finalized, we only target 2.2 because the 3.0
+ * defines etc are still changing, so users may have an older version
+ * of the 3.0 headers lying around, which may prevent clinfo from being
+ * compilable.
+ */
+#define CL_TARGET_OPENCL_VERSION 220
+
 /* We will use the deprecated clGetExtensionFunctionAddress,
  * so let the headers know that we don't care about it being deprecated.
  * The standard CL_USE_DEPRECATED_OPENCL_1_1_APIS define apparently
@@ -24,6 +36,58 @@
 #include <CL/cl.h>
 #endif
 
+/* Very old headers will be missing these defines */
+#ifndef CL_VERSION_1_1
+#define CL_DEVICE_PREFERRED_VECTOR_WIDTH_HALF           0x1034
+#define CL_DEVICE_HOST_UNIFIED_MEMORY                   0x1035
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_CHAR              0x1036
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_SHORT             0x1037
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_INT               0x1038
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_LONG              0x1039
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_FLOAT             0x103A
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_DOUBLE            0x103B
+#define CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF              0x103C
+#define CL_DEVICE_OPENCL_C_VERSION                      0x103D
+
+#define CL_FP_SOFT_FLOAT                                (1 << 6)
+
+#define CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE    0x11B3
+#endif
+
+#ifndef CL_VERSION_1_2
+#define CL_DEVICE_TYPE_CUSTOM                           (1 << 4)
+
+#define CL_DEVICE_LINKER_AVAILABLE                      0x103E
+#define CL_DEVICE_BUILT_IN_KERNELS                      0x103F
+#define CL_DEVICE_IMAGE_MAX_BUFFER_SIZE                 0x1040
+#define CL_DEVICE_IMAGE_MAX_ARRAY_SIZE                  0x1041
+#define CL_DEVICE_PARTITION_MAX_SUB_DEVICES             0x1043
+#define CL_DEVICE_PARTITION_PROPERTIES                  0x1044
+#define CL_DEVICE_PARTITION_AFFINITY_DOMAIN             0x1045
+#define CL_DEVICE_PARTITION_TYPE                        0x1046
+#define CL_DEVICE_PREFERRED_INTEROP_USER_SYNC           0x1048
+#define CL_DEVICE_PRINTF_BUFFER_SIZE                    0x1049
+#define CL_DEVICE_IMAGE_PITCH_ALIGNMENT                 0x104A
+#define CL_DEVICE_IMAGE_BASE_ADDRESS_ALIGNMENT          0x104B
+
+#define CL_FP_CORRECTLY_ROUNDED_DIVIDE_SQRT             (1 << 7)
+
+/* cl_device_partition_property */
+#define CL_DEVICE_PARTITION_EQUALLY                     0x1086
+#define CL_DEVICE_PARTITION_BY_COUNTS                   0x1087
+#define CL_DEVICE_PARTITION_BY_COUNTS_LIST_END          0x0
+#define CL_DEVICE_PARTITION_BY_AFFINITY_DOMAIN          0x1088
+
+/* cl_device_affinity_domain */
+#define CL_DEVICE_AFFINITY_DOMAIN_NUMA                  (1 << 0)
+#define CL_DEVICE_AFFINITY_DOMAIN_L4_CACHE              (1 << 1)
+#define CL_DEVICE_AFFINITY_DOMAIN_L3_CACHE              (1 << 2)
+#define CL_DEVICE_AFFINITY_DOMAIN_L2_CACHE              (1 << 3)
+#define CL_DEVICE_AFFINITY_DOMAIN_L1_CACHE              (1 << 4)
+#define CL_DEVICE_AFFINITY_DOMAIN_NEXT_PARTITIONABLE    (1 << 5)
+
+#endif
+
 /* These two defines were introduced in the 1.2 headers
  * on 2012-11-30, so earlier versions don't have them
  * (e.g. Debian wheezy)
@@ -71,6 +135,52 @@ typedef cl_bitfield         cl_device_svm_capabilities;
 #define CL_DEVICE_SUB_GROUP_INDEPENDENT_FORWARD_PROGRESS 0x105D
 #endif
 
+#ifndef CL_VERSION_3_0
+#define CL_PLATFORM_NUMERIC_VERSION			0x0906
+#define CL_PLATFORM_EXTENSIONS_WITH_VERSION		0x0907
+#define CL_DEVICE_NUMERIC_VERSION			0x105E
+#define CL_DEVICE_EXTENSIONS_WITH_VERSION		0x1060
+#define CL_DEVICE_ILS_WITH_VERSION			0x1061
+#define CL_DEVICE_BUILT_IN_KERNELS_WITH_VERSION		0x1062
+#define CL_DEVICE_ATOMIC_MEMORY_CAPABILITIES		0x1063
+#define CL_DEVICE_ATOMIC_FENCE_CAPABILITIES		0x1064
+#define CL_DEVICE_NON_UNIFORM_WORK_GROUP_SUPPORT	0x1065
+#define CL_DEVICE_OPENCL_C_ALL_VERSIONS			0x1066
+#define CL_DEVICE_PREFERRED_WORK_GROUP_SIZE_MULTIPLE	0x1067
+#define CL_DEVICE_WORK_GROUP_COLLECTIVE_FUNCTIONS_SUPPORT 0x1068
+#define CL_DEVICE_GENERIC_ADDRESS_SPACE_SUPPORT		0x1069
+#define CL_DEVICE_OPENCL_C_FEATURES			0x106F
+#define CL_DEVICE_DEVICE_ENQUEUE_CAPABILITIES		0x1070
+#define CL_DEVICE_PIPE_SUPPORT				0x1071
+#define CL_DEVICE_LATEST_CONFORMANCE_VERSION_PASSED	0x1072
+
+
+typedef cl_bitfield	cl_device_atomic_capabilities;
+typedef cl_bitfield	cl_device_device_enqueue_capabilities;
+typedef cl_uint		cl_version;
+
+#define CL_NAME_VERSION_MAX_NAME_SIZE 64
+
+typedef struct _cl_name_version {
+    cl_version              version;
+    char                    name[CL_NAME_VERSION_MAX_NAME_SIZE];
+} cl_name_version;
+
+/* cl_device_atomic_capabilities */
+#define CL_DEVICE_ATOMIC_ORDER_RELAXED		(1 << 0)
+#define CL_DEVICE_ATOMIC_ORDER_ACQ_REL		(1 << 1)
+#define CL_DEVICE_ATOMIC_ORDER_SEQ_CST		(1 << 2)
+#define CL_DEVICE_ATOMIC_SCOPE_WORK_ITEM	(1 << 3)
+#define CL_DEVICE_ATOMIC_SCOPE_WORK_GROUP	(1 << 4)
+#define CL_DEVICE_ATOMIC_SCOPE_DEVICE		(1 << 5)
+#define CL_DEVICE_ATOMIC_SCOPE_ALL_DEVICES	(1 << 6)
+
+/* cl_device_device_enqueue_capabilities */
+#define CL_DEVICE_QUEUE_SUPPORTED               (1 << 0)
+#define CL_DEVICE_QUEUE_REPLACEABLE_DEFAULT     (1 << 1)
+
+#endif
+
 /*
  * Extensions
  */
@@ -82,6 +192,17 @@ typedef cl_bitfield         cl_device_svm_capabilities;
 /* cl_amd_object_metadata */
 #define CL_PLATFORM_MAX_KEYS_AMD			0x403C
 
+/* cl_khr_device_uuid extension */
+
+#define CL_UUID_SIZE_KHR 16
+#define CL_LUID_SIZE_KHR 8
+
+#define CL_DEVICE_UUID_KHR          0x106A
+#define CL_DRIVER_UUID_KHR          0x106B
+#define CL_DEVICE_LUID_VALID_KHR    0x106C
+#define CL_DEVICE_LUID_KHR          0x106D
+#define CL_DEVICE_NODE_MASK_KHR     0x106E
+
 /* cl_khr_fp64 */
 #define CL_DEVICE_DOUBLE_FP_CONFIG			0x1032
 
@@ -166,6 +287,9 @@ typedef union
 #define CL_DEVICE_NUM_P2P_DEVICES_AMD			0x4088
 #define CL_DEVICE_P2P_DEVICES_AMD			0x4089
 
+/* cl_ext_cxx_for_opencl */
+#define CL_DEVICE_CXX_FOR_OPENCL_NUMERIC_VERSION_EXT	0x4230
+
 /* cl_ext_device_fission */
 #define cl_ext_device_fission				1
 
@@ -209,6 +333,9 @@ typedef cl_ulong  cl_device_partition_property_ext;
 /* cl_arm_shared_virtual_memory */
 #define CL_DEVICE_SVM_CAPABILITIES_ARM			0x40B6
 
+/* cl_arm_core_id */
+#define CL_DEVICE_COMPUTE_UNITS_BITFIELD_ARM		0x40BF
+
 /* cl_khr_spir */
 #define CL_DEVICE_SPIR_VERSIONS				0x40E0
 
@@ -222,6 +349,9 @@ typedef cl_ulong  cl_device_partition_property_ext;
 /* cl_intel_required_subgroup_size */
 #define CL_DEVICE_SUB_GROUP_SIZES_INTEL			0x4108
 
+/* cl_arm_job_slot_selection */
+#define CL_DEVICE_JOB_SLOTS_ARM				0x41E0
+
 /* clGeICDLoaderInfoOCLICD */
 typedef enum {
 	CL_ICDL_OCL_VERSION=1,
